import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.user_organization_role import UserOrganizationRole
from ..models.user_status import UserStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.deployment_role import DeploymentRole
    from ..models.workspace_role import WorkspaceRole


T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """
    Attributes:
        avatar_url (str): The URL for the user's profile image. Example: https://avatar.url.
        created_at (datetime.datetime): The time when the user was created in UTC, formatted as `YYYY-MM-DDTHH:MM:SSZ`.
            Example: 2022-11-22T04:37:12Z.
        full_name (str): The user's full name. Example: Jane Doe.
        id (str): The user's ID. Example: clm9sq6s0000008kz7uvl7yz7.
        status (UserStatus): The user's status. Example: ACTIVE.
        updated_at (datetime.datetime): The time when the user was updated in UTC, formatted as `YYYY-MM-DDTHH:MM:SSZ`.
            Example: 2022-11-22T04:37:12Z.
        username (str): The user's username. Example: user1@company.com.
        deployment_roles (Union[Unset, list['DeploymentRole']]): The user's Deployment roles.
        organization_role (Union[Unset, UserOrganizationRole]): The user's Organization role. Example:
            ORGANIZATION_MEMBER.
        workspace_roles (Union[Unset, list['WorkspaceRole']]): The user's Workspace roles.
    """

    avatar_url: str
    created_at: datetime.datetime
    full_name: str
    id: str
    status: UserStatus
    updated_at: datetime.datetime
    username: str
    deployment_roles: Union[Unset, list["DeploymentRole"]] = UNSET
    organization_role: Union[Unset, UserOrganizationRole] = UNSET
    workspace_roles: Union[Unset, list["WorkspaceRole"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        avatar_url = self.avatar_url

        created_at = self.created_at.isoformat()

        full_name = self.full_name

        id = self.id

        status = self.status.value

        updated_at = self.updated_at.isoformat()

        username = self.username

        deployment_roles: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.deployment_roles, Unset):
            deployment_roles = []
            for deployment_roles_item_data in self.deployment_roles:
                deployment_roles_item = deployment_roles_item_data.to_dict()
                deployment_roles.append(deployment_roles_item)

        organization_role: Union[Unset, str] = UNSET
        if not isinstance(self.organization_role, Unset):
            organization_role = self.organization_role.value

        workspace_roles: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.workspace_roles, Unset):
            workspace_roles = []
            for workspace_roles_item_data in self.workspace_roles:
                workspace_roles_item = workspace_roles_item_data.to_dict()
                workspace_roles.append(workspace_roles_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "avatarUrl": avatar_url,
                "createdAt": created_at,
                "fullName": full_name,
                "id": id,
                "status": status,
                "updatedAt": updated_at,
                "username": username,
            }
        )
        if deployment_roles is not UNSET:
            field_dict["deploymentRoles"] = deployment_roles
        if organization_role is not UNSET:
            field_dict["organizationRole"] = organization_role
        if workspace_roles is not UNSET:
            field_dict["workspaceRoles"] = workspace_roles

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.deployment_role import DeploymentRole
        from ..models.workspace_role import WorkspaceRole

        d = dict(src_dict)
        avatar_url = d.pop("avatarUrl")

        created_at = isoparse(d.pop("createdAt"))

        full_name = d.pop("fullName")

        id = d.pop("id")

        status = UserStatus(d.pop("status"))

        updated_at = isoparse(d.pop("updatedAt"))

        username = d.pop("username")

        deployment_roles = []
        _deployment_roles = d.pop("deploymentRoles", UNSET)
        for deployment_roles_item_data in _deployment_roles or []:
            deployment_roles_item = DeploymentRole.from_dict(deployment_roles_item_data)

            deployment_roles.append(deployment_roles_item)

        _organization_role = d.pop("organizationRole", UNSET)
        organization_role: Union[Unset, UserOrganizationRole]
        if isinstance(_organization_role, Unset):
            organization_role = UNSET
        else:
            organization_role = UserOrganizationRole(_organization_role)

        workspace_roles = []
        _workspace_roles = d.pop("workspaceRoles", UNSET)
        for workspace_roles_item_data in _workspace_roles or []:
            workspace_roles_item = WorkspaceRole.from_dict(workspace_roles_item_data)

            workspace_roles.append(workspace_roles_item)

        user = cls(
            avatar_url=avatar_url,
            created_at=created_at,
            full_name=full_name,
            id=id,
            status=status,
            updated_at=updated_at,
            username=username,
            deployment_roles=deployment_roles,
            organization_role=organization_role,
            workspace_roles=workspace_roles,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class WorkspaceRoleRole(str, Enum):
    WORKSPACE_ACCESSOR = "WORKSPACE_ACCESSOR"
    WORKSPACE_AUTHOR = "WORKSPACE_AUTHOR"
    WORKSPACE_MEMBER = "WORKSPACE_MEMBER"
    WORKSPACE_OPERATOR = "WORKSPACE_OPERATOR"
    WORKSPACE_OWNER = "WORKSPACE_OWNER"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.default_role import DefaultRole
    from ..models.role import Role


T = TypeVar("T", bound="RolesPaginated")


@_attrs_define
class RolesPaginated:
    """
    Attributes:
        limit (int): The number of custom roles returned. Example: 1.
        offset (int): The offset of the custom roles. Example: 1.
        roles (list['Role']): The list of custom roles.
        total_count (int): The total number of custom roles. Example: 1.
        default_roles (Union[Unset, list['DefaultRole']]): The list of default roles.
    """

    limit: int
    offset: int
    roles: list["Role"]
    total_count: int
    default_roles: Union[Unset, list["DefaultRole"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        limit = self.limit

        offset = self.offset

        roles = []
        for roles_item_data in self.roles:
            roles_item = roles_item_data.to_dict()
            roles.append(roles_item)

        total_count = self.total_count

        default_roles: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.default_roles, Unset):
            default_roles = []
            for default_roles_item_data in self.default_roles:
                default_roles_item = default_roles_item_data.to_dict()
                default_roles.append(default_roles_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "limit": limit,
                "offset": offset,
                "roles": roles,
                "totalCount": total_count,
            }
        )
        if default_roles is not UNSET:
            field_dict["defaultRoles"] = default_roles

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.default_role import DefaultRole
        from ..models.role import Role

        d = dict(src_dict)
        limit = d.pop("limit")

        offset = d.pop("offset")

        roles = []
        _roles = d.pop("roles")
        for roles_item_data in _roles:
            roles_item = Role.from_dict(roles_item_data)

            roles.append(roles_item)

        total_count = d.pop("totalCount")

        default_roles = []
        _default_roles = d.pop("defaultRoles", UNSET)
        for default_roles_item_data in _default_roles or []:
            default_roles_item = DefaultRole.from_dict(default_roles_item_data)

            default_roles.append(default_roles_item)

        roles_paginated = cls(
            limit=limit,
            offset=offset,
            roles=roles,
            total_count=total_count,
            default_roles=default_roles,
        )

        roles_paginated.additional_properties = d
        return roles_paginated

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.team_organization_role import TeamOrganizationRole
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.basic_subject_profile import BasicSubjectProfile
    from ..models.deployment_role import DeploymentRole
    from ..models.workspace_role import WorkspaceRole


T = TypeVar("T", bound="Team")


@_attrs_define
class Team:
    """
    Attributes:
        created_at (datetime.datetime): The time when the Team was created in UTC, formatted as `YYYY-MM-DDTHH:MM:SSZ`.
            Example: 2022-11-22T04:37:12Z.
        id (str): The Team's ID. Example: clma5ftgk000008mhgev00k7d.
        is_idp_managed (bool): Whether the Team is managed by an identity provider (IdP).
        name (str): The Team's name. Example: My Team.
        organization_id (str): The ID of the Organization to which the Team belongs. Example: clma5g8q6000108mh88g27k1y.
        organization_role (TeamOrganizationRole): The Team's Organization role. Example: ORGANIZATION_MEMBER.
        updated_at (datetime.datetime): The time when the Team was last updated in UTC, formatted as `YYYY-MM-
            DDTHH:MM:SSZ`. Example: 2022-11-22T04:37:12Z.
        created_by (Union[Unset, BasicSubjectProfile]):
        deployment_roles (Union[Unset, list['DeploymentRole']]): The Team's role in each Deployment it belongs to.
        description (Union[Unset, str]): The Team's description. Example: My Team description.
        roles_count (Union[Unset, int]): The number of roles the Team has. Example: 1.
        updated_by (Union[Unset, BasicSubjectProfile]):
        workspace_roles (Union[Unset, list['WorkspaceRole']]): The Team's role in each Workspace it belongs to.
    """

    created_at: datetime.datetime
    id: str
    is_idp_managed: bool
    name: str
    organization_id: str
    organization_role: TeamOrganizationRole
    updated_at: datetime.datetime
    created_by: Union[Unset, "BasicSubjectProfile"] = UNSET
    deployment_roles: Union[Unset, list["DeploymentRole"]] = UNSET
    description: Union[Unset, str] = UNSET
    roles_count: Union[Unset, int] = UNSET
    updated_by: Union[Unset, "BasicSubjectProfile"] = UNSET
    workspace_roles: Union[Unset, list["WorkspaceRole"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        id = self.id

        is_idp_managed = self.is_idp_managed

        name = self.name

        organization_id = self.organization_id

        organization_role = self.organization_role.value

        updated_at = self.updated_at.isoformat()

        created_by: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.created_by, Unset):
            created_by = self.created_by.to_dict()

        deployment_roles: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.deployment_roles, Unset):
            deployment_roles = []
            for deployment_roles_item_data in self.deployment_roles:
                deployment_roles_item = deployment_roles_item_data.to_dict()
                deployment_roles.append(deployment_roles_item)

        description = self.description

        roles_count = self.roles_count

        updated_by: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.updated_by, Unset):
            updated_by = self.updated_by.to_dict()

        workspace_roles: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.workspace_roles, Unset):
            workspace_roles = []
            for workspace_roles_item_data in self.workspace_roles:
                workspace_roles_item = workspace_roles_item_data.to_dict()
                workspace_roles.append(workspace_roles_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "id": id,
                "isIdpManaged": is_idp_managed,
                "name": name,
                "organizationId": organization_id,
                "organizationRole": organization_role,
                "updatedAt": updated_at,
            }
        )
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if deployment_roles is not UNSET:
            field_dict["deploymentRoles"] = deployment_roles
        if description is not UNSET:
            field_dict["description"] = description
        if roles_count is not UNSET:
            field_dict["rolesCount"] = roles_count
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by
        if workspace_roles is not UNSET:
            field_dict["workspaceRoles"] = workspace_roles

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.basic_subject_profile import BasicSubjectProfile
        from ..models.deployment_role import DeploymentRole
        from ..models.workspace_role import WorkspaceRole

        d = dict(src_dict)
        created_at = isoparse(d.pop("createdAt"))

        id = d.pop("id")

        is_idp_managed = d.pop("isIdpManaged")

        name = d.pop("name")

        organization_id = d.pop("organizationId")

        organization_role = TeamOrganizationRole(d.pop("organizationRole"))

        updated_at = isoparse(d.pop("updatedAt"))

        _created_by = d.pop("createdBy", UNSET)
        created_by: Union[Unset, BasicSubjectProfile]
        if isinstance(_created_by, Unset):
            created_by = UNSET
        else:
            created_by = BasicSubjectProfile.from_dict(_created_by)

        deployment_roles = []
        _deployment_roles = d.pop("deploymentRoles", UNSET)
        for deployment_roles_item_data in _deployment_roles or []:
            deployment_roles_item = DeploymentRole.from_dict(deployment_roles_item_data)

            deployment_roles.append(deployment_roles_item)

        description = d.pop("description", UNSET)

        roles_count = d.pop("rolesCount", UNSET)

        _updated_by = d.pop("updatedBy", UNSET)
        updated_by: Union[Unset, BasicSubjectProfile]
        if isinstance(_updated_by, Unset):
            updated_by = UNSET
        else:
            updated_by = BasicSubjectProfile.from_dict(_updated_by)

        workspace_roles = []
        _workspace_roles = d.pop("workspaceRoles", UNSET)
        for workspace_roles_item_data in _workspace_roles or []:
            workspace_roles_item = WorkspaceRole.from_dict(workspace_roles_item_data)

            workspace_roles.append(workspace_roles_item)

        team = cls(
            created_at=created_at,
            id=id,
            is_idp_managed=is_idp_managed,
            name=name,
            organization_id=organization_id,
            organization_role=organization_role,
            updated_at=updated_at,
            created_by=created_by,
            deployment_roles=deployment_roles,
            description=description,
            roles_count=roles_count,
            updated_by=updated_by,
            workspace_roles=workspace_roles,
        )

        team.additional_properties = d
        return team

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.api_token_type import ApiTokenType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.api_token_role import ApiTokenRole
    from ..models.basic_subject_profile import BasicSubjectProfile


T = TypeVar("T", bound="ApiToken")


@_attrs_define
class ApiToken:
    """
    Attributes:
        created_at (datetime.datetime): The time when the API token was created in UTC, formatted as `YYYY-MM-
            DDTHH:MM:SSZ`. Example: 2022-11-22T04:37:12Z.
        description (str): The description of the API token. Example: my token description.
        id (str): The API token's ID. Example: clm8q7f6q000008lcgyougpsk.
        name (str): The name of the API token. Example: My token.
        short_token (str): The short value of the API token. Example: short-token.
        start_at (datetime.datetime): The time when the API token will become valid in UTC, formatted as `YYYY-MM-
            DDTHH:MM:SSZ`. Example: 2022-11-22T04:37:12Z.
        type_ (ApiTokenType): The type of the API token. Example: WORKSPACE.
        updated_at (datetime.datetime): The time when the API token was last updated in UTC, formatted as `YYYY-MM-
            DDTHH:MM:SSZ`. Example: 2022-11-22T04:37:12Z.
        created_by (Union[Unset, BasicSubjectProfile]):
        end_at (Union[Unset, datetime.datetime]): The time when the API token expires in UTC, formatted as `YYYY-MM-
            DDTHH:MM:SSZ`. Example: 2022-11-22T04:37:12Z.
        expiry_period_in_days (Union[Unset, int]): The expiry period of the API token in days. Example: 30.
        last_used_at (Union[Unset, datetime.datetime]): The time when the API token was last used in UTC, formatted as
            `YYYY-MM-DDTHH:MM:SSZ`. Example: 2022-11-22T04:37:12Z.
        roles (Union[Unset, list['ApiTokenRole']]): The roles of the API token.
        token (Union[Unset, str]): The value of the API token. Example: token.
        updated_by (Union[Unset, BasicSubjectProfile]):
    """

    created_at: datetime.datetime
    description: str
    id: str
    name: str
    short_token: str
    start_at: datetime.datetime
    type_: ApiTokenType
    updated_at: datetime.datetime
    created_by: Union[Unset, "BasicSubjectProfile"] = UNSET
    end_at: Union[Unset, datetime.datetime] = UNSET
    expiry_period_in_days: Union[Unset, int] = UNSET
    last_used_at: Union[Unset, datetime.datetime] = UNSET
    roles: Union[Unset, list["ApiTokenRole"]] = UNSET
    token: Union[Unset, str] = UNSET
    updated_by: Union[Unset, "BasicSubjectProfile"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        description = self.description

        id = self.id

        name = self.name

        short_token = self.short_token

        start_at = self.start_at.isoformat()

        type_ = self.type_.value

        updated_at = self.updated_at.isoformat()

        created_by: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.created_by, Unset):
            created_by = self.created_by.to_dict()

        end_at: Union[Unset, str] = UNSET
        if not isinstance(self.end_at, Unset):
            end_at = self.end_at.isoformat()

        expiry_period_in_days = self.expiry_period_in_days

        last_used_at: Union[Unset, str] = UNSET
        if not isinstance(self.last_used_at, Unset):
            last_used_at = self.last_used_at.isoformat()

        roles: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.roles, Unset):
            roles = []
            for roles_item_data in self.roles:
                roles_item = roles_item_data.to_dict()
                roles.append(roles_item)

        token = self.token

        updated_by: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.updated_by, Unset):
            updated_by = self.updated_by.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "description": description,
                "id": id,
                "name": name,
                "shortToken": short_token,
                "startAt": start_at,
                "type": type_,
                "updatedAt": updated_at,
            }
        )
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if end_at is not UNSET:
            field_dict["endAt"] = end_at
        if expiry_period_in_days is not UNSET:
            field_dict["expiryPeriodInDays"] = expiry_period_in_days
        if last_used_at is not UNSET:
            field_dict["lastUsedAt"] = last_used_at
        if roles is not UNSET:
            field_dict["roles"] = roles
        if token is not UNSET:
            field_dict["token"] = token
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.api_token_role import ApiTokenRole
        from ..models.basic_subject_profile import BasicSubjectProfile

        d = dict(src_dict)
        created_at = isoparse(d.pop("createdAt"))

        description = d.pop("description")

        id = d.pop("id")

        name = d.pop("name")

        short_token = d.pop("shortToken")

        start_at = isoparse(d.pop("startAt"))

        type_ = ApiTokenType(d.pop("type"))

        updated_at = isoparse(d.pop("updatedAt"))

        _created_by = d.pop("createdBy", UNSET)
        created_by: Union[Unset, BasicSubjectProfile]
        if isinstance(_created_by, Unset):
            created_by = UNSET
        else:
            created_by = BasicSubjectProfile.from_dict(_created_by)

        _end_at = d.pop("endAt", UNSET)
        end_at: Union[Unset, datetime.datetime]
        if isinstance(_end_at, Unset):
            end_at = UNSET
        else:
            end_at = isoparse(_end_at)

        expiry_period_in_days = d.pop("expiryPeriodInDays", UNSET)

        _last_used_at = d.pop("lastUsedAt", UNSET)
        last_used_at: Union[Unset, datetime.datetime]
        if isinstance(_last_used_at, Unset):
            last_used_at = UNSET
        else:
            last_used_at = isoparse(_last_used_at)

        roles = []
        _roles = d.pop("roles", UNSET)
        for roles_item_data in _roles or []:
            roles_item = ApiTokenRole.from_dict(roles_item_data)

            roles.append(roles_item)

        token = d.pop("token", UNSET)

        _updated_by = d.pop("updatedBy", UNSET)
        updated_by: Union[Unset, BasicSubjectProfile]
        if isinstance(_updated_by, Unset):
            updated_by = UNSET
        else:
            updated_by = BasicSubjectProfile.from_dict(_updated_by)

        api_token = cls(
            created_at=created_at,
            description=description,
            id=id,
            name=name,
            short_token=short_token,
            start_at=start_at,
            type_=type_,
            updated_at=updated_at,
            created_by=created_by,
            end_at=end_at,
            expiry_period_in_days=expiry_period_in_days,
            last_used_at=last_used_at,
            roles=roles,
            token=token,
            updated_by=updated_by,
        )

        api_token.additional_properties = d
        return api_token

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="TeamMember")


@_attrs_define
class TeamMember:
    """
    Attributes:
        user_id (str): The Team member's ID. Example: clma5vzk2000108k20jhq3f7n.
        username (str): The Team member's username. Example: user1@company.com.
        avatar_url (Union[Unset, str]): The URL for the Team member's profile image. Example: https://avatar.url.
        created_at (Union[Unset, datetime.datetime]): The time when the Team member was added in UTC, formatted as
            `YYYY-MM-DDTHH:MM:SSZ`. Example: 2022-11-22T04:37:12Z.
        full_name (Union[Unset, str]): The Team member's full name. Example: Jane Doe.
    """

    user_id: str
    username: str
    avatar_url: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    full_name: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        user_id = self.user_id

        username = self.username

        avatar_url = self.avatar_url

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        full_name = self.full_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "userId": user_id,
                "username": username,
            }
        )
        if avatar_url is not UNSET:
            field_dict["avatarUrl"] = avatar_url
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if full_name is not UNSET:
            field_dict["fullName"] = full_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        user_id = d.pop("userId")

        username = d.pop("username")

        avatar_url = d.pop("avatarUrl", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        full_name = d.pop("fullName", UNSET)

        team_member = cls(
            user_id=user_id,
            username=username,
            avatar_url=avatar_url,
            created_at=created_at,
            full_name=full_name,
        )

        team_member.additional_properties = d
        return team_member

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.basic_subject_profile import BasicSubjectProfile


T = TypeVar("T", bound="AllowedIpAddressRange")


@_attrs_define
class AllowedIpAddressRange:
    """
    Attributes:
        created_at (datetime.datetime): The time when the allowed IP address range was created in UTC, formatted as
            `YYYY-MM-DDTHH:MM:SSZ`. Example: 2022-11-22T04:37:12Z.
        id (str): The allowed IP address range's ID. Example: clm9sq6s0000008kz7uvl7yz7.
        ip_address_range (str): The allowed IP address range in CIDR format. Example: 1.1.1.1/32.
        organization_id (str): The  allowed IP address range's Organization ID. Example: clyt27999000008me3yp39wcp.
        updated_at (datetime.datetime): The time when the allowed IP address range was updated in UTC, formatted as
            `YYYY-MM-DDTHH:MM:SSZ`. Example: 2022-11-22T04:37:12Z.
        created_by (Union[Unset, BasicSubjectProfile]):
        updated_by (Union[Unset, BasicSubjectProfile]):
    """

    created_at: datetime.datetime
    id: str
    ip_address_range: str
    organization_id: str
    updated_at: datetime.datetime
    created_by: Union[Unset, "BasicSubjectProfile"] = UNSET
    updated_by: Union[Unset, "BasicSubjectProfile"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        id = self.id

        ip_address_range = self.ip_address_range

        organization_id = self.organization_id

        updated_at = self.updated_at.isoformat()

        created_by: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.created_by, Unset):
            created_by = self.created_by.to_dict()

        updated_by: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.updated_by, Unset):
            updated_by = self.updated_by.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "id": id,
                "ipAddressRange": ip_address_range,
                "organizationId": organization_id,
                "updatedAt": updated_at,
            }
        )
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.basic_subject_profile import BasicSubjectProfile

        d = dict(src_dict)
        created_at = isoparse(d.pop("createdAt"))

        id = d.pop("id")

        ip_address_range = d.pop("ipAddressRange")

        organization_id = d.pop("organizationId")

        updated_at = isoparse(d.pop("updatedAt"))

        _created_by = d.pop("createdBy", UNSET)
        created_by: Union[Unset, BasicSubjectProfile]
        if isinstance(_created_by, Unset):
            created_by = UNSET
        else:
            created_by = BasicSubjectProfile.from_dict(_created_by)

        _updated_by = d.pop("updatedBy", UNSET)
        updated_by: Union[Unset, BasicSubjectProfile]
        if isinstance(_updated_by, Unset):
            updated_by = UNSET
        else:
            updated_by = BasicSubjectProfile.from_dict(_updated_by)

        allowed_ip_address_range = cls(
            created_at=created_at,
            id=id,
            ip_address_range=ip_address_range,
            organization_id=organization_id,
            updated_at=updated_at,
            created_by=created_by,
            updated_by=updated_by,
        )

        allowed_ip_address_range.additional_properties = d
        return allowed_ip_address_range

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListAllowedIpAddressRangesSortsItem(str, Enum):
    CREATEDATASC = "createdAt:asc"
    CREATEDATDESC = "createdAt:desc"
    IPADDRESSASC = "ipAddress:asc"
    IPADDRESSDESC = "ipAddress:desc"
    UPDATEDATASC = "updatedAt:asc"
    UPDATEDATDESC = "updatedAt:desc"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UserStatus(str, Enum):
    ACTIVE = "ACTIVE"
    BANNED = "BANNED"
    INACTIVE = "INACTIVE"
    PENDING = "PENDING"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class DefaultRoleScopeType(str, Enum):
    DEPLOYMENT = "DEPLOYMENT"
    ORGANIZATION = "ORGANIZATION"
    SYSTEM = "SYSTEM"
    WORKSPACE = "WORKSPACE"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ListTeamMembersSortsItem(str, Enum):
    CREATEDATASC = "createdAt:asc"
    CREATEDATDESC = "createdAt:desc"
    FULLNAMEASC = "fullName:asc"
    FULLNAMEDESC = "fullName:desc"
    USERIDASC = "userId:asc"
    USERIDDESC = "userId:desc"
    USERNAMEASC = "username:asc"
    USERNAMEDESC = "username:desc"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.update_team_roles_request_organization_role import UpdateTeamRolesRequestOrganizationRole
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.deployment_role import DeploymentRole
    from ..models.workspace_role import WorkspaceRole


T = TypeVar("T", bound="UpdateTeamRolesRequest")


@_attrs_define
class UpdateTeamRolesRequest:
    """
    Attributes:
        organization_role (UpdateTeamRolesRequestOrganizationRole): The Team's Organization roles. Example:
            ORGANIZATION_MEMBER.
        deployment_roles (Union[Unset, list['DeploymentRole']]): The user's updated Deployment roles. The Deployments
            you specify must belong to the Team's Organization.
        workspace_roles (Union[Unset, list['WorkspaceRole']]): The Team's updated Workspace roles. The Workspaces you
            specify must belong to the Team's Organization.
    """

    organization_role: UpdateTeamRolesRequestOrganizationRole
    deployment_roles: Union[Unset, list["DeploymentRole"]] = UNSET
    workspace_roles: Union[Unset, list["WorkspaceRole"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        organization_role = self.organization_role.value

        deployment_roles: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.deployment_roles, Unset):
            deployment_roles = []
            for deployment_roles_item_data in self.deployment_roles:
                deployment_roles_item = deployment_roles_item_data.to_dict()
                deployment_roles.append(deployment_roles_item)

        workspace_roles: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.workspace_roles, Unset):
            workspace_roles = []
            for workspace_roles_item_data in self.workspace_roles:
                workspace_roles_item = workspace_roles_item_data.to_dict()
                workspace_roles.append(workspace_roles_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "organizationRole": organization_role,
            }
        )
        if deployment_roles is not UNSET:
            field_dict["deploymentRoles"] = deployment_roles
        if workspace_roles is not UNSET:
            field_dict["workspaceRoles"] = workspace_roles

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.deployment_role import DeploymentRole
        from ..models.workspace_role import WorkspaceRole

        d = dict(src_dict)
        organization_role = UpdateTeamRolesRequestOrganizationRole(d.pop("organizationRole"))

        deployment_roles = []
        _deployment_roles = d.pop("deploymentRoles", UNSET)
        for deployment_roles_item_data in _deployment_roles or []:
            deployment_roles_item = DeploymentRole.from_dict(deployment_roles_item_data)

            deployment_roles.append(deployment_roles_item)

        workspace_roles = []
        _workspace_roles = d.pop("workspaceRoles", UNSET)
        for workspace_roles_item_data in _workspace_roles or []:
            workspace_roles_item = WorkspaceRole.from_dict(workspace_roles_item_data)

            workspace_roles.append(workspace_roles_item)

        update_team_roles_request = cls(
            organization_role=organization_role,
            deployment_roles=deployment_roles,
            workspace_roles=workspace_roles,
        )

        update_team_roles_request.additional_properties = d
        return update_team_roles_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
